@@ -7,12 +7,34 @@ from __future__ import unicode_literals
 
 import contextlib
 import io
+import os
+import subprocess
+import sys
 import tempfile
 import unittest
 
 import pyformat
 
 
+ROOT_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
+
+
+if (
+    'PYFORMAT_COVERAGE' in os.environ and
+    int(os.environ['PYFORMAT_COVERAGE'])
+):
+    PYFORMAT_COMMAND = ['coverage', 'run', '--branch', '--parallel',
+                        '--omit=*/distutils/*,*/site-packages/*',
+                        os.path.join(ROOT_DIRECTORY, 'pyformat.py')]
+else:
+    # We need to specify the executable to make sure the correct Python
+    # interpreter gets used.
+    PYFORMAT_COMMAND = [sys.executable,
+                        os.path.join(
+                            ROOT_DIRECTORY,
+                            'pyformat.py')]  # pragma: no cover
+
+
 class TestUnits(unittest.TestCase):
 
     def test_format_code(self):
@@ -46,9 +68,9 @@ import os
 x = "abc"
 ''') as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             self.assertEqual('''\
 @@ -1,2 +1,2 @@
  import os
@@ -62,9 +84,9 @@ import os
 x = "abc"
 ''') as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', '--aggressive', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', '--aggressive', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             self.assertEqual('''\
 @@ -1,2 +1 @@
 -import os
@@ -75,9 +97,9 @@ x = "abc"
     def test_diff_with_empty_file(self):
         with temporary_file('') as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             self.assertEqual('', output_file.getvalue())
 
     def test_diff_with_encoding_declaration(self):
@@ -89,9 +111,9 @@ import my_own_module
 x = 1
 """) as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', '--aggressive', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', '--aggressive', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             self.assertEqual("""\
  # coding: utf-8
 -import re
@@ -102,16 +124,16 @@ x = 1
 
     def test_diff_with_nonexistent_file(self):
         output_file = io.StringIO()
-        pyformat.main(argv=['my_fake_program', 'nonexistent_file'],
-                      standard_out=output_file,
-                      standard_error=output_file)
+        pyformat._main(argv=['my_fake_program', 'nonexistent_file'],
+                       standard_out=output_file,
+                       standard_error=output_file)
         self.assertIn('no such file', output_file.getvalue().lower())
 
     def test_verbose(self):
         output_file = io.StringIO()
-        pyformat.main(argv=['my_fake_program', '--verbose', __file__],
-                      standard_out=output_file,
-                      standard_error=output_file)
+        pyformat._main(argv=['my_fake_program', '--verbose', __file__],
+                       standard_out=output_file,
+                       standard_error=output_file)
         self.assertIn('.py', output_file.getvalue())
 
     def test_in_place(self):
@@ -120,9 +142,9 @@ if True:
     x = "abc"
 ''') as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', '--in-place', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', '--in-place', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             with open(filename) as f:
                 self.assertEqual('''\
 if True:
@@ -135,10 +157,10 @@ if True:
     x = "abc"
 ''') as filename:
             output_file = io.StringIO()
-            pyformat.main(argv=['my_fake_program', '--in-place',
-                                '--jobs=2', filename],
-                          standard_out=output_file,
-                          standard_error=None)
+            pyformat._main(argv=['my_fake_program', '--in-place',
+                                 '--jobs=2', filename],
+                           standard_out=output_file,
+                           standard_error=None)
             with open(filename) as f:
                 self.assertEqual('''\
 if True:
@@ -147,10 +169,10 @@ if True:
 
     def test_multiple_jobs_should_require_in_place(self):
         output_file = io.StringIO()
-        pyformat.main(argv=['my_fake_program',
-                            '--jobs=2', __file__],
-                      standard_out=output_file,
-                      standard_error=output_file)
+        pyformat._main(argv=['my_fake_program',
+                             '--jobs=2', __file__],
+                       standard_out=output_file,
+                       standard_error=output_file)
 
         self.assertIn('requires --in-place', output_file.getvalue())
 
@@ -171,11 +193,11 @@ if True:
 """, directory=inner_directory):
 
                     output_file = io.StringIO()
-                    pyformat.main(argv=['my_fake_program',
-                                        '--recursive',
-                                        directory],
-                                  standard_out=output_file,
-                                  standard_error=None)
+                    pyformat._main(argv=['my_fake_program',
+                                         '--recursive',
+                                         directory],
+                                   standard_out=output_file,
+                                   standard_error=None)
                     self.assertEqual(
                         '',
                         output_file.getvalue().strip())
@@ -188,12 +210,13 @@ if True:
 """, prefix='food', directory=directory):
 
                 output_file = io.StringIO()
-                pyformat.main(argv=['my_fake_program',
-                                    '--recursive',
-                                    '--exclude=zap,x*oo*',
-                                    directory],
-                              standard_out=output_file,
-                              standard_error=None)
+                pyformat._main(argv=['my_fake_program',
+                                     '--recursive',
+                                     '--exclude=zap',
+                                     '--exclude=x*oo*',
+                                     directory],
+                               standard_out=output_file,
+                               standard_error=None)
                 self.assertEqual("""\
 @@ -1,2 +1,2 @@
  if True:
@@ -209,16 +232,30 @@ if True:
 """, prefix='food', directory=directory):
 
                 output_file = io.StringIO()
-                pyformat.main(argv=['my_fake_program',
-                                    '--recursive',
-                                    '--exclude=zap,*oo*',
-                                    directory],
-                              standard_out=output_file,
-                              standard_error=None)
+                pyformat._main(argv=['my_fake_program',
+                                     '--recursive',
+                                     '--exclude=zap',
+                                     '--exclude=*oo*',
+                                     directory],
+                               standard_out=output_file,
+                               standard_error=None)
                 self.assertEqual(
                     '',
                     output_file.getvalue().strip())
 
+    def test_end_to_end(self):
+        with temporary_file("""\
+import os
+x = "abc"
+""") as filename:
+            process = subprocess.Popen(PYFORMAT_COMMAND + [filename],
+                                       stdout=subprocess.PIPE)
+            self.assertEqual("""\
+ import os
+-x = "abc"
++x = 'abc'
+""", '\n'.join(process.communicate()[0].decode('utf-8').split('\n')[3:]))
+
 
 @contextlib.contextmanager
 def temporary_file(contents, directory='.', prefix=''):
